uniform sampler2D CurrentGrid;
uniform sampler2D Transformation;

varying vec2 neighborhoodTex0;
varying vec2 neighborhoodTex1;
varying vec2 neighborhoodTex2;
varying vec2 neighborhoodTex3;
varying vec2 neighborhoodTex4;


const float base=9.0;
const float states=5.0;
const float shift=729.0;
const float twidth=81.0;

vec4 neighborhood[9];
vec4 target;

void checkRules() {
   neighborhood[0]=ceil(states*texture2D(CurrentGrid, neighborhoodTex0));
   neighborhood[1]=ceil(states*texture2D(CurrentGrid, neighborhoodTex1));
   neighborhood[2]=ceil(states*texture2D(CurrentGrid, neighborhoodTex2));
   neighborhood[3]=ceil(states*texture2D(CurrentGrid, neighborhoodTex3));
   neighborhood[4]=ceil(states*texture2D(CurrentGrid, neighborhoodTex4));

   vec4 current;
   
   float r=neighborhood[2].r;
   float g=neighborhood[2].g;
   float b=neighborhood[2].b;
   float a=neighborhood[2].a;

   //now check for each cell in current the rules
   //top left
   float hash=
			pow(base,neighborhood[0].b)+
			pow(base,neighborhood[1].g)+
			pow(base,b)+
			pow(base,g);
            
   float tx=floor(hash/shift);
   float ty=hash-tx*shift;           
   vec2 ttc=vec2((tx+r*twidth)*2.0,ty)/1024.0;
   current=texture2D(Transformation,ttc);
   target.r=current.r;

   //top right
   hash=
			pow(base,neighborhood[0].a)+
			pow(base,neighborhood[3].r)+
			pow(base,r)+
			pow(base,a);
   
   tx=floor(hash/shift);
   ty=hash-tx*shift;           
   ttc=vec2((tx+g*twidth)*2.0,ty)/1024.0;
   current=texture2D(Transformation,ttc);
   target.g=current.r;

   //bottom left
   hash=
            pow(base,neighborhood[1].a)+
            pow(base,neighborhood[4].r)+
            pow(base,r)+
            pow(base,a);
   
   tx=floor(hash/shift);
   ty=hash-tx*shift;           
   ttc=vec2((tx+b*twidth)*2.0,ty)/1024.0;
   current=texture2D(Transformation,ttc);
   target.b=current.r;
   
   //bottom right
   hash=
            pow(base,neighborhood[3].b)+
            pow(base,neighborhood[4].g)+
            pow(base,g)+
            pow(base,b);
   
   tx=floor(hash/shift);
   ty=hash-tx*shift;           
   ttc=vec2((tx+a*twidth)*2.0,ty)/1024.0;
   current=texture2D(Transformation,ttc);
   target.a=current.r;
   
   gl_FragColor=target;   
}

void main(void)
{
   checkRules();  
}